function [P,U,V] = compute_PUV(A,B,outdim)
% -------------Added by caicai-------------
%% compute the projections: Linear Discriminant Projections (P and U) and
%% PCA
% [P,U,V] = compute_PUV(A,B,outdim)
%
%% INPUT:
% A     -- inter-class covariance (M*M, from unmatched feature difference)
% B     -- intra-class covariance (M*M, from matched feature difference)
% outdim-- how many projective vectors for output (default: M)
%
%% OUTPUT:
% P     -- Linear discriminant projection P (outdim*M, one row corresponds to one
%          projective vector)
% U     -- Linear discriminant projection U (outdim*M, one row corresponds to one
%          projective vector)
% V     -- PCA projection (outdim*M, one row corresponds to one projective vector)
%
% ----------- 14th, Nov, 2009 --------------
if nargin<3
    outdim = size(A);
end;
if outdim>size(A)
    error(' The outdim should not be larger than the size of A!');
end

%==V( for PCA)
[eigvector,D] = eig(A);
eigvalue = spdiags(D);
[eigvalue,index] = sort(-eigvalue);
eigvalue = -eigvalue;
eigvector =  eigvector(:,index(1:outdim));
V = eigvector';

%==P
B_12 = sqrtm(B);
B_12 = inv(B_12);
tem = B_12*A*B_12;
[eigvector,D] = eig(tem);
eigvalue = spdiags(D);
[eigvalue,index] = sort(-eigvalue);
eigvalue = -eigvalue;
eigvector =  eigvector(:,index(1:outdim));
Vec = eigvector;
P = Vec'*B_12;

%==U
ww = inv(B)*A;
[eigvector,D] = eig(ww);
eigvalue = spdiags(D);
[eigvalue,index] = sort(-eigvalue);
eigvalue = -eigvalue;
eigvector = eigvector(:,index(1:outdim));
U = eigvector';
end