function n_rg = detect_dense_regions(img_file,delta,r,detect_file)
% -------------Added by caicai-------------
%% detect dense regions in an image
%
% n_rg = detect_dense_regions(img_file,delta,r,detect_file)
%
%% INPUT:
% img_file   -- image file name
% delta      -- region sampling intervals(1*n vectors)
%  r         -- corresponding region radii(1*n vectors),delta(i) corresponds
%             to r(i)
% detect_file  -- output text file name
%  <<detect_file format>>
% 1
% n
% x_1 y_1 a_1 b_1 c_1
%      :
%      :
% x_n y_1 a_n b_n c_n
%
% x,y,a,b,c are 5 parameters for an ellipse:  a(x-u)(x-u)+2b(x-u)(y-v)+c(y-v)(y-v)=1
% For a circle with radius r, a = 1/r^2, b = 0, c = 1/r^2.
%
%% OUTPUT:
%  n_rg    -- How many regions are detected
% 
%% How to use:
% Get regions in the image with sampling intervals are [8 15],
% corresponding radii are [15 20]
% detect_grid_group('image1.png',[8 15],[15 20],'image1.dense');
%%
% --------------14th Nov 2009----------------

if length(delta)~= length(r)
    error('The delta and r should be the same length.');
end;
im = imread(img_file);
h = size(im,1);
w = size(im,2);

all_feat = [];
for i=1:length(delta)
    y = ceil(r(i))+1:delta(i):h-ceil(r(i))-1;
    x = ceil(r(i))+1:delta(i):w-ceil(r(i))-1;
    [grid_y,grid_x] = meshgrid(y,x);

    n_rg = length(grid_x(:));
    vector_r = ones(n_rg,1).*r(i);
    a = 1./(vector_r.^2);
    feat = [grid_x(:) grid_y(:) a zeros(n_rg,1) a vector_r];
    all_feat = [all_feat;feat];
end
n_rg = size(all_feat,1);
fidi = fopen(detect_file,'wt');
fprintf(fidi,'%d', 0);
fprintf(fidi,'\n');
fprintf(fidi,'%d',n_rg);
fprintf(fidi,'\n');
fprintf(fidi,'%f %f %f %d %f\n',all_feat(:,1:5)');
fclose(fidi);
  
