function [diff_feat,ind] = diff_rotate(im_file,feat_uniq,sigma_degree,ntimes,tmp_file1,tmp_file2,b_inside,flag)
% -----------added by caicai----------------
%% simulate the differences of matched SIFT features with rotation transform
%
% [diff_feat,ind] = diff_rotate(im_file,feat_uniq,sigma_degree,ntimes,tmp_file1,tmp_file2,b_inside,flag)
%
%% INPUT:
%  im_file -- image file name (*.png)
% feat_uniq-- n m-dim original image features (n*m,no  location repeated regions)
% sigma_degree-standard deviation for Gaussian distributed angle change   (flag=1)
%             maximal scale changes for uniform distributed angle change
%             i.e., randomly selected a param [-sigma_degree sigma_degree](flag=0) 
%  ntimes  -- How many times for the simulation
% tmp_file1-- Output temporary file for *.detector
% tmp_file2-- Output temporary file for *.detector.sift
% b_inside -- Are all regions should be inside regions, i.e., not on the boundary of the images
%             (1--inside,0--all regions used) (default: b_inside=1), not
%             used in this function.
%  flag    -- 0: random angles with uniform distribution
%             1: random angles with Gaussian distribution
%             (default: flag=0)
%
%% OUTPUT:
% diff_feat-- a matrix of feature differences, each row is one difference
%              vector
% ind      -- the output region indeces (1:nb in this function)
%
%% NOTE:
% No two regions in the input features locate the same position!!
% -------------14th Nov, 2009-----------------

if nargin<7
    b_inside = 1;    
end
if nargin<8
    flag = 0;
end;

rand('twister', sum(100*clock));
im = imread(im_file);
h = size(im,1);
w = size(im,2);

nb = size(feat_uniq,1);
b = sigma_degree*pi/180;
a = -b;
        
feat_rr = feat_uniq(:,3);
diff_feat = [];

k = findstr(im_file, '/');
kk = k(end);
tmp_imfile = [im_file(1:kk) 'tmp' int2str(rand(1)) im_file(kk+1:end)];
for i=1:ntimes
    fprintf(1,' %d.',i);
    if flag == 0
        fai = a+(b-a)*rand(1);
    else
        fai = b*randn(1);
    end;
    tmpim = imrotate(im,fai*180/pi,'bilinear');
    imwrite(tmpim,tmp_imfile);
        
    r = 1./sqrt(feat_rr);
    feat_x = feat_uniq(:,1);
    feat_y = feat_uniq(:,2);
    feat_xnew = (feat_x-1).*cos(-fai)-(feat_y-1).*sin(-fai)+1;
    feat_ynew = (feat_y-1).*cos(-fai)+(feat_x-1).*sin(-fai)+1;    
    if sin(fai)<0
        feat_xnew = feat_xnew-h.*sin(fai);
    else
        feat_ynew = feat_ynew+w.*sin(fai);
    end;
    
    feat_locnew = [feat_xnew feat_ynew feat_uniq(:,3:5)];
    ind = 1:nb;    
    feat_org = feat_uniq(ind,:);
    writelocFeatures(feat_locnew,tmp_file1);
    tmp_outfile = [tmp_file1 '.txt'];
    command = sprintf('./desc_vidi_64_angles.ln %d %s %s %s>%s',0,tmp_imfile,tmp_file1,tmp_file2,tmp_outfile);
    system(command);  
    delete(tmp_outfile);
    [feat2,nb2,dim] = loadFeatures(tmp_file2);
    feat2 = feat2';  
    
    if size(feat_org,1) == size(feat2,1)
        cur_diff_feat = feat_org(:,6:end)-feat2(:,6:end);
    else
        error('Make sure that no two regions in the input features locate the same position!!');
    end; 
    diff_feat = [diff_feat;cur_diff_feat];    
end
fprintf(1,'\n');
delete(tmp_imfile);
end

function writelocFeatures(feat,file_txt)
fidi = fopen(file_txt,'wt');
fprintf(fidi,'%d', 0);
fprintf(fidi,'\n');
fprintf(fidi,'%d',size(feat,1));
fprintf(fidi,'\n');
fprintf(fidi,'%f %f %f %f %f\n',feat');
fclose(fidi);
end
