function [diff_feat,ind] = diff_scale(im_file,feat_uniq,sigma_scale,ntimes,tmp_file1,tmp_file2,b_inside,flag)
% -----------added by caicai----------------
%% simulate the differences of matched SIFT features with scale transform
%
% [diff_feat,ind] = diff_scale(im_file,feat_uniq,sigma_scale,ntimes,tmp_file1,tmp_file2,b_inside)
%
%% INPUT:
% im_file  -- image file name(*.png)
% feat_uniq-- n m-dim original image features(n*m,no location repeated regions)
% sigma_scale-standard deviation for Gaussian distributed scale change   (flag=1)
%             maximal scale changes for uniform distributed scale change
%             i.e., randomly selected a param [1/sigma_scale sigma_scale](flag=0) 
% ntimes   -- How many times for the simulation
% tmp_file1-- Output temporary file for *.detector
% tmp_file2-- Output temporary file for *.detector.sift
% b_inside -- Are all regions should be inside regions, i.e., not on the boundary of the images
%            (1--inside,0--all regions used) (default: b_inside=1)
%  flag    -- 0: the random scales with uniform distribution
%             1: the random scales with Gaussian distribution
%             (default: flag=0)
%
%% OUTPUT:
% diff_feat-- a matrix of feature differences, each row is one difference
%             vector
% ind      -- the output index (for some regions are not included for
%             differnece computing)
%
%% NOTE:
% No two regions in the input features locate the same position!!
% -------------14th Nov, 2009-----------------

if nargin<7
    b_inside = 1;    
end
if nargin<8
    flag = 0;
end;

im = imread(im_file);
h = size(im,1);
w = size(im,2);
nb = size(feat_uniq,1);
b = log(sigma_scale);
a = -b;        
feat_rr = feat_uniq(:,3);
diff_feat = [];
for i=1:ntimes
    fprintf(1,' %d.',i);
    if flag == 0
        s = a+(b-a)*rand(nb,1);
    else
        s = b*randn(nb,1);
    end;
    s = exp(s);    
    r = 1./sqrt(feat_rr);
    new_r = r.*s;
    feat_rrnew = 1./(new_r.^2);
    feat_locnew = [feat_uniq(:,1:2) feat_rrnew zeros(nb,1) feat_rrnew];
    
    if b_inside
        r = new_r;
        xmin = feat_uniq(:,1)-r;
        ymin = feat_uniq(:,2)-r;
        xmax = feat_uniq(:,1)+r;
        ymax = feat_uniq(:,2)+r;
        ind = find((xmin>0)&(ymin>0)&(xmax<w)&(ymax<h));
    else
        ind = 1:nb;
    end;
    
    if length(ind)>0
        feat_org = feat_uniq(ind,:);
        feat_locnew = feat_locnew(ind,:);
        writelocFeatures(feat_locnew,tmp_file1);
        tmp_outfile = [tmp_file1 '.txt'];
        command = sprintf('./desc_vidi_64_angles.ln %d %s %s %s>%s',0,im_file,tmp_file1,tmp_file2,tmp_outfile);
        system(command);
        delete(tmp_outfile);
        [feat2,nb2,dim] = loadFeatures(tmp_file2);
        feat2 = feat2';
       
        if size(feat_org,1) == size(feat2,1)
            cur_diff_feat = feat_org(:,6:end)-feat2(:,6:end);
        else
            error('Make sure that no two regions in the input features locate the same position!!');       
        end;
    else
        cur_diff_feat = [];
    end;
    diff_feat = [diff_feat;cur_diff_feat];
end
fprintf(1,'\n');
end

function writelocFeatures(feat,file_txt)
fidi = fopen(file_txt,'wt');
fprintf(fidi,'%d', 0);
fprintf(fidi,'\n');
fprintf(fidi,'%d',size(feat,1));
fprintf(fidi,'\n');
fprintf(fidi,'%f %f %f %f %f\n',feat');
fclose(fidi);
end
