function [diff_feat,ind] = diff_translate(im_file,feat_uniq,sigma_trans,ntimes,tmp_file1,tmp_file2,b_inside,flag)
% -----------added by caicai----------------
%% simulate the differences of matched SIFT features with tranlation transform
%
% [diff_feat,ind] = diff_translate(im_file,feat_uniq,sigma_trans,ntimes,tmp_file1,tmp_file2,b_inside,flag)
%
%% INPUT:
%  im_file -- image file name(*.png)
% feat_uniq-- n m-dim original image features(n*m,no location repeated regions)
% sigma_trans-standard deviation for Gaussian distributed translation changes   (flag=1)
%             maximal translation pixels for uniform distributed
%             translation changes
%             i.e., randomly selected a param [-sigma_trans sigma_trans] (flag=0) 
% ntimes   -- How many times for the simulation
%tmp_file1 -- Output temporary file for *.detector
%tmp_file2 -- Output temporary file for *.detector.sift
% b_inside -- Are all regions should be inside regions, i.e., not on the boundary of the images
%            (1--inside,0--all regions used) (default: b_inside=1)
%  flag    -- 0: the random shifts with uniform distribution
%             1: the random shifts with Gaussian distribution
%             (default: flag=0)
%
%% OUTPUT:
% diff_feat-- a matrix of feature differences, each row is one difference
% vector
% ind      -- the output index(for some regions are not included for
% differnece computing)
%
%% NOTE:
% No two regions in the input features locate the same position!!
% -------------14th Nov, 2009-----------------

if nargin<7
    b_inside = 1;
end
if nargin<8
    flag = 0;
end;

im = imread(im_file);
h = size(im,1);
w = size(im,2);
nb = size(feat_uniq,1);
a = -sigma_trans;
b = sigma_trans;

feat_xy = feat_uniq(:,1:2);
diff_feat = [];
for i=1:ntimes
    fprintf(1,' %d.',i);
    if flag == 0
        s = a+(b-a).*rand(nb,2);
    else
        s = b*randn(nb,2);
    end;
    feat_xynew = feat_xy + s;
    feat_locnew = [feat_xynew feat_uniq(:,3:5)];    
    
    if b_inside
        r = 1./sqrt(feat_uniq(:,3));
        xmin = feat_xynew(:,1)-r;
        ymin = feat_xynew(:,2)-r;
        xmax = feat_xynew(:,1)+r;
        ymax = feat_xynew(:,2)+r;
        ind = find((xmin>0)&(ymin>0)&(xmax<w)&(ymax<h));
    else
        ind = 1:nb;
    end;
    
    if length(ind) >0
        feat_org = feat_uniq(ind,:);
        feat_locnew = feat_locnew(ind,:);
        writelocFeatures(feat_locnew,tmp_file1);
        tmp_outfile = [tmp_file1 '.txt'];
        command = sprintf('./desc_vidi_64_angles.ln %d %s %s %s>%s',0,im_file,tmp_file1,tmp_file2,tmp_outfile);
        system(command);
        delete(tmp_outfile);
        [feat2,nb2,dim] = loadFeatures(tmp_file2);
        feat2 = feat2';

        if size(feat_org,1) == size(feat2,1)
            cur_diff_feat = feat_org(:,6:end)-feat2(:,6:end);
        else
            error('Make sure that no two regions in the input features locate the same position!!');
        end;
    else
        cur_diff_feat = [];
    end;
    diff_feat = [diff_feat;cur_diff_feat];
end
fprintf(1,'\n');
end

function writelocFeatures(feat,file_txt)
fidi = fopen(file_txt,'wt');
fprintf(fidi,'%d', 0);
fprintf(fidi,'\n');
fprintf(fidi,'%d',size(feat,1));
fprintf(fidi,'\n');
fprintf(fidi,'%f %f %f %f %f\n',feat');
fclose(fidi);
end

    
    
    
    
